/*
 *  This file is part of BGX, the Bayesian Gene eXpression program.
 *  Copyright (c) 2008 Ernest Turro <ernest.turro@ic.ac.uk>
 *
 *  BGX is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  BGX is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#include <algorithm>
#include <map>
#include <ctype.h>
#include <cstdlib>
#include <cstring>
#ifdef WIN32
 #include <io.h>
#else
  #include <sys/stat.h>
#endif
#include <dirent.h>
#include <string>
#include "file/TsvFile/PgfFile.h"
#include <iostream>
#include <fstream>
#include <iomanip>

using namespace std;
using namespace affx;

extern "C" {

void genestructParser(char **path_to_pgf, int * ignore_probeset_ids, int *no_to_ignore, char **input_dir) {

  // Try opening PGF file
  PgfFile* pgf = new PgfFile();
  if ((pgf->open(string(*path_to_pgf)))!=TSV_OK) {
    printf("Error: couldn't open PGF file \"%s\"\n",*path_to_pgf);
    delete pgf;
    return;
  }

  // Try opening the input file for BGX
  DIR* input_dir_=opendir(*input_dir);
  char a = 'y';
  if(input_dir_ != NULL){
    a = 'n';
    printf("Input dir \"%s\" already exists. Overwrite? (y/N) ", *input_dir);
    scanf("%c", &a);
    a = tolower(a);
  }
  if(a=='y') {
#ifdef WIN32
    mkdir(*input_dir);
#else
    mkdir(*input_dir,S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
#endif
  } else {
    exit(1);
  }
    
  map<const int, int> probeId2Index;
  vector<vector <int> > PS2PM;
  vector<vector <int> > PM2PS;
  vector<vector <int> > PM2PM;
  vector<vector <double> > PM2PMOverlap;
  vector<int> PMids;
  vector<int> PMcat;
  
  vector<int> BGcat;
  vector<int> BGids;

  vector<int> PSids;

  int j=0; // unique probe counter
  int g=0; // main gene counter

//  ofstream seqs("sequences.txt");


  printf("Parsing Probe Group file... ");
  fflush(stdout);

  vector<int> haps;
  for(int i=0; i < *no_to_ignore; i++) {
    haps.push_back(ignore_probeset_ids[i]);
  }

  while(pgf->next_probeset()==TSV_OK){
    if(pgf->probeset_type=="main" && find(haps.begin(), haps.end(), pgf->probeset_id) == haps.end()) {
      PSids.push_back(pgf->probeset_id);
      vector<int> pr;
      while(pgf->next_atom()==TSV_OK){
        while (pgf->next_probe()==TSV_OK) {
          if(probeId2Index.count(pgf->probe_id)==0) {
     //       seqs << pgf->probe_sequence << endl;
            probeId2Index[pgf->probe_id] = j;
            PM2PS.resize(j+1);
            PM2PS[j].push_back(g);
            pr.push_back(j);
            PMids.push_back(pgf->probe_id);
            PMcat.push_back(pgf->gc_count);
            j++;
          } else {
            PM2PS[probeId2Index[pgf->probe_id]].push_back(g);
            pr.push_back(probeId2Index[pgf->probe_id]);
          }
        }
      }
      PS2PM.push_back(pr);
      //cout << g << '\r' << flush;
      g++;
    } else if(pgf->probeset_type=="control->bgp->antigenomic") {
      while(pgf->next_atom()==TSV_OK){
        while (pgf->next_probe()==TSV_OK) {
          BGcat.push_back(pgf->gc_count);
          BGids.push_back(pgf->probe_id);
        }
      }
    }
  }

  for(int g=0; g < (int)PS2PM.size(); g++) {
    vector<int> ge;
    vector<double> geOverlap;
    for(int p=0; p < (int)PS2PM[g].size(); p++) {
      for(int h=0; h < (int)PM2PS[PS2PM[g][p]].size(); h++) {
        if(find(ge.begin(), ge.end(), PM2PS[PS2PM[g][p]][h]) == ge.end()) {
          ge.push_back(PM2PS[PS2PM[g][p]][h]);
          vector<int> v1 = PS2PM[g], v2 = PS2PM[PM2PS[PS2PM[g][p]][h]];
          vector<int> res(PS2PM[g].size() + PS2PM[PM2PS[PS2PM[g][p]][h]].size()); 
          vector<int>::iterator it;
          sort(v1.begin(), v1.end()); sort(v2.begin(), v2.end());
          it = set_intersection(v1.begin(), v1.end(), v2.begin(), v2.end(), res.begin());
          geOverlap.push_back((double)(it - res.begin())/PS2PM[g].size());
        }
      }
    }
    PM2PM.push_back(ge);
    PM2PMOverlap.push_back(geOverlap);
  }


  printf("done\n");

  printf("Found %d probesets, %d unique probes.\n", (int)PS2PM.size(), (int)PM2PS.size());

  ofstream of((string(*input_dir) + '/' + "PM2PS.txt").c_str());
  for(int i=0; i < (int)PM2PS.size(); i++) {
    for(int j=0; j < (int)PM2PS[i].size(); j++) {
      of << PM2PS[i][j] << " ";
    }
    of << endl;
  }
  of.close();
  of.open((string(*input_dir) + '/' + "PS2PM.txt").c_str());
  for(int i=0; i < (int)PS2PM.size(); i++) {
    for(int j=0; j < (int)PS2PM[i].size(); j++) {
      of << PS2PM[i][j] << " ";
    }
    of << endl;
  }
  of.close();
  of.open((string(*input_dir) + '/' + "PMids.txt").c_str());
  for(int i=0; i < (int)PMids.size(); i++) of << PMids[i] << endl;
  of.close();
  of.open((string(*input_dir) + '/' + "PSids.txt").c_str());
  for(int i=0; i < (int)PSids.size(); i++) of << PSids[i] << endl;
  of.close();
  of.open((string(*input_dir) + '/' + "BGids.txt").c_str());
  for(int i=0; i < (int)BGids.size(); i++) of << BGids[i] << endl;
  of.close();
  of.open((string(*input_dir) + '/' + "BGcats.txt").c_str());
  for(int i=0; i < (int)BGcat.size(); i++) of << BGcat[i] << endl;
  of.close();
  of.open((string(*input_dir) + '/' + "PMcats.txt").c_str());
  for(int i=0; i < (int)PMcat.size(); i++) of << PMcat[i] << endl;
  of.close();
  of.open((string(*input_dir) + '/' + "PM2PM.txt").c_str());
  for(int i=0; i < (int)PM2PM.size(); i++) {
    for(int j=0; j < (int)PM2PM[i].size(); j++) {
      of << PM2PM[i][j] << "(" << setprecision(4) << PM2PMOverlap[i][j] << ")\t";
    }
    of << endl;
  }
  of.close();

  pgf->close();
  delete pgf;
}
}

/* int main() {
  char *path_to_pgf = new char[256];
  strcpy(path_to_pgf, "/Users/telos/Desktop/HuGene-1_0-st-v1.r3.analysis_libraryfile/HuGene-1_0-st-v1.r3.pgf");
  int *ignore_probeset_ids = new int[2];
  ignore_probeset_ids[0] =  8177462 ;
  ignore_probeset_ids[1] = 8177478 ;
  int no_to_ignore = 2;
  char *input_dir = new char[256];
  strcpy(input_dir, "foobar-input");

  parsePGF(&path_to_pgf, ignore_probeset_ids, &no_to_ignore, &input_dir); 

  return 0;
} */
